// This has the declaration of a Fish class.  Fish can be drawn in a graphics
// context in a variety of colors. They can face left, or right, and can move, 
// reverseDirection.
// ASSUMES: the background where the fish is has color Blue.

// written by Scott McElfresh

import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.geom.*;

public class Fish 
{
  
  private int x,y;  // location of nose of fish
  private int width,height;  // body of fish
  private boolean facingLeft;
  private Color theColor;
  private JFrame f;
  
  private static Random RNG = new Random();  // random number generators for fish
  								// "static" means that all fish share the same RNG
  

  // *same as above, but no randomness for colors
  public Fish(int x, int y, String color)
  {
        
     this.x = x;
     this.y = y;
     
    width = RNG.nextInt(50) + 20;
    height = RNG.nextInt(20)+10;
    if (RNG.nextInt(2) == 0)
    	facingLeft = true;
    else 
         facingLeft = false;
         
    theColor = FishColors.convert(color); 
  }
  //******
  // accessors here
  public int getX()
  {  return x;
  }
  public int getY()
  {  return y;
  }
  public String getColor()
  {
    return  FishColors.convert(theColor);
  }
  public boolean getFacingLeft()
  {  return facingLeft;
  }
  //*****
  // tell the fish what context it is in 
  public void setFrame(JFrame f)
  {
    this.f = f;;
  }
  //********************
  public void draw()
  {
    if (wayoffscreen())
    {
        System.out.println("FISH ATTEMPTED TO BE DRAWN OFF SCREEN.");
        System.exit(1);
    }
   	Graphics2D theContext = (Graphics2D)f.getGraphics(); 
    theContext.setColor(theColor);
   		
    // Graphics theContext = f.getGraphics();
    // theContext.setColor(theColor);
     if (facingLeft)
     {  // draw the body
        Ellipse2D.Double body = new Ellipse2D.Double(x,y-height/2,width,height);
        theContext.fill(body);
        //theContext.fillOval(x,y-height/2,width,height);
        // draw the tail
        Polygon tail = new Polygon();
        tail.addPoint(x+width-2,y);
        tail.addPoint(x+width+5, y-height/2);
        tail.addPoint(x+width+5,y +height/2 );
		theContext.fill(tail);
		// draw the eye
		Ellipse2D.Double eye = new Ellipse2D.Double(x + 5, y -3, 3,3);
		theContext.setColor(Color.black);
		theContext.fill(eye);
     }
     else
     {  // draw the body
        Ellipse2D.Double body = new Ellipse2D.Double(x-width,y-height/2,width,height);
        theContext.fill(body);
        //theContext.fillOval(x-width,y-height/2,width,height);
        // draw the tail
        Polygon tail = new Polygon();
        tail.addPoint(x-width+2,y);
        tail.addPoint(x-width-5, y-height/2);
        tail.addPoint(x-width-5,y +height/2 );
		theContext.fill(tail);
		// draw the eye
	    Ellipse2D.Double eye = new Ellipse2D.Double(x - 5, y -3, 3,3);
		theContext.setColor(Color.black);
		theContext.fill(eye);
     }
     
     
  }
  //********
  // This causes the fish to reverse its direction.
  public void reverseDirection()
  {
      erase();
      facingLeft = ! facingLeft;
      draw();
  }
  
  ///*************
  // remove fish from view by drawing over in Blue.  A slightly larger
  // rectangle is drawn so some surrounding pixels will also turn Blue.
  public void erase()
  {    
   	Graphics2D theContext = (Graphics2D)f.getGraphics(); 
     if (facingLeft)
     {  theContext.setColor(Color.blue);
        Rectangle2D.Double box = new Rectangle2D.Double(x,y - height/2,width+6,height+1);
        theContext.fill(box);
     }
     else
     {  
       theContext.setColor(Color.blue);
       Rectangle2D.Double box = new Rectangle2D.Double(x -width-5,y - height/2,width+6,height+1);
       theContext.fill(box);
     }
  }
  
  
  //****************
  // move the fish on screen by one pixel
  public void move()
  {
    erase();
    if (facingLeft)
      x --;
    else
      x++;
      
    draw();
  }
  
  
  //******************
  // determine if the fish is still on screen or not
  public boolean visible()
  {
  int x1 = 0;
  int x2 = f.getWidth();
    if (facingLeft)
    {
        if (x + width + 5 < x1)
           return false;
        if (x > x2)
           return false;
    }
    else  // !facingLeft
    {
        if (x < x1)
          return false;
        if (x -width - 5 > x2)
           return false;
    }
    return true;
  }
  
  //******************
  // determine if the fish is WAY OFF SCREEN (more than 100 pixels)
  private boolean wayoffscreen()
  {
     if (visible())
       return false;
     int x1 = 0;
     int x2 = f.getWidth();
     if (x < x1 - 100)
        return true;
     if (x > x2 + 100)
        return true;
      return false;
  }
}